﻿using Infrastructure;
using Infrastructure.Attribute;
using Infrastructure.Enums;
using Infrastructure.Model;
using Mapster;
using Microsoft.AspNetCore.Mvc;
using ZR.Model.Dto;
using ZR.Model.Models;
using ZR.Service.Business.IBusinessService;
using ZR.Admin.WebApi.Extensions;
using ZR.Admin.WebApi.Filters;
using ZR.Common;

namespace ZR.Admin.WebApi.Controllers
{
    /// <summary>
    /// 入库管理/入库记录表Controller
    /// 
    /// @tableName in_instore_record
    /// @author admin
    /// @date 2023-04-13
    /// </summary>
    [Verify]
    [Route("business/InInstoreRecord")]
    public class InInstoreRecordController : BaseController
    {
        /// <summary>
        /// 入库管理/入库记录表接口
        /// </summary>
        private readonly IInInstoreRecordService _InInstoreRecordService;

        public InInstoreRecordController(IInInstoreRecordService InInstoreRecordService)
        {
            _InInstoreRecordService = InInstoreRecordService;
        }

        /// <summary>
        /// 查询入库管理/入库记录表列表
        /// </summary>
        /// <param name="parm"></param>
        /// <returns></returns>
        [HttpGet("list")]
        [ActionPermissionFilter(Permission = "business:ininstorerecord:list")]
        public IActionResult QueryInInstoreRecord([FromQuery] InInstoreRecordQueryDto parm)
        {
            var response = _InInstoreRecordService.GetList(parm);
            return SUCCESS(response);
        }


        /// <summary>
        /// 查询入库管理/入库记录表详情
        /// </summary>
        /// <param name="InStoreId"></param>
        /// <returns></returns>
        [HttpGet("{InStoreId}")]
        [ActionPermissionFilter(Permission = "business:ininstorerecord:query")]
        public IActionResult GetInInstoreRecord(int InStoreId)
        {
            var response = _InInstoreRecordService.GetFirst(x => x.InStoreId == InStoreId);

            return SUCCESS(response);
        }

        /// <summary>
        /// 添加入库管理/入库记录表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [ActionPermissionFilter(Permission = "business:ininstorerecord:add")]
        [Log(Title = "入库管理/入库记录表", BusinessType = BusinessType.INSERT)]
        public IActionResult AddInInstoreRecord([FromBody] InInstoreRecordDto parm)
        {
            if (parm == null)
            {
                throw new CustomException("请求参数错误");
            }
            var modal = parm.Adapt<InInstoreRecord>().ToCreate(HttpContext);

            var response = _InInstoreRecordService.AddInInstoreRecord(modal);

            return ToResponse(response);
        }

        /// <summary>
        /// 更新入库管理/入库记录表
        /// </summary>
        /// <returns></returns>
        [HttpPut]
        [ActionPermissionFilter(Permission = "business:ininstorerecord:edit")]
        [Log(Title = "入库管理/入库记录表", BusinessType = BusinessType.UPDATE)]
        public IActionResult UpdateInInstoreRecord([FromBody] InInstoreRecordDto parm)
        {
            if (parm == null)
            {
                throw new CustomException("请求实体不能为空");
            }
            var modal = parm.Adapt<InInstoreRecord>().ToUpdate(HttpContext);

            var response = _InInstoreRecordService.UpdateInInstoreRecord(modal);

            return ToResponse(response);
        }

        /// <summary>
        /// 删除入库管理/入库记录表
        /// </summary>
        /// <returns></returns>
        [HttpDelete("{ids}")]
        [ActionPermissionFilter(Permission = "business:ininstorerecord:delete")]
        [Log(Title = "入库管理/入库记录表", BusinessType = BusinessType.DELETE)]
        public IActionResult DeleteInInstoreRecord(string ids)
        {
            int[] idsArr = Tools.SpitIntArrary(ids);
            if (idsArr.Length <= 0) { return ToResponse(ApiResult.Error($"删除失败Id 不能为空")); }

            var response = _InInstoreRecordService.Delete(idsArr);

            return ToResponse(response);
        }



    }
}